{% extends "admin/base.html" %}
{% load i18n static %}

{% block title %}Reversion Compare Test | {{ title }}{% endblock %}

{% block branding %}
<h1 id="site-name">
    <a href="{% url 'admin:index' %}">
        Reversion Compare Test Admin
    </a>
</h1>
{% endblock %}

{% block nav-global %}{% endblock %}
